@@ -2,17 +2,23 @@
 package OOPS::Upgrade::To1003;
 
 use OOPS::Setup;
+use strict;
+use warnings;
 
 sub upgrade
 {
-	my ($oops, $oldversion) = @_;
+	my ($oldversion, %args) = @_;
 
 	die unless $oldversion eq '1001';
 
 	print STDERR "# Schema upgrade to 1003...\n" if $OOPS::debug_upgrade;
 
-	if ($oops->{dbms} eq 'mysql') {
-		$oops->db_domany($oops->{args}, <<END);
+	my $dbo = OOPS::DBO->dboconnect(%args);
+	my $dbh = $dbo->{dbh};
+	my $prefix = $dbo->{table_prefix};
+
+	if ($dbo->{dbms} eq 'mysql') {
+		$dbo->db_domany(<<END, args => \%args);
 			
 			ALTER TABLE TP_attribute 
 			MODIFY COLUMN pkey VARCHAR(255) BINARY;
@@ -20,15 +26,16 @@ sub upgrade
 			ALTER TABLE TP_big
 			MODIFY COLUMN pkey VARCHAR(255) BINARY;
 END
+		$dbo->commit();
 	}
-	my (@r) = $oops->db_domany($oops->{args}, <<END);
+	$dbo->db_domany(<<END, args => \%args);
 
 		UPDATE TP_object
 		SET alen = 1003
 		WHERE id = 1;
 
 END
-	$oops->{arraylen}{1} = '1003';	# in case it is saved
+	$dbo->commit();
 }
 
 1;
